//
//  ViewController.swift
//  Citizen Label sample 1
//
//  Created by Citizen on 2018/04/09
//  Copyright (c) 2018年 CITIZEN. All rights reserved.
//

import UIKit
import CSJLabelLibSwift

class ViewController: UIViewController, UITextFieldDelegate {

    @IBOutlet weak var segmentPortType: UISegmentedControl!
    @IBOutlet weak var editText_Address_Wifi: UITextField!
    
    // Types of errors
    enum ERRORKIND
    {
        case ERR_CONNECT
        case ERR_PRINTERCHECK
        case ERR_PRINT
        case ERR_PRINTERSTATUS
    }

    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        
        editText_Address_Wifi.returnKeyType = UIReturnKeyType.done
        editText_Address_Wifi.delegate = self
        
        segmentPortType.selectedSegmentIndex = 0
        
        editText_Address_Wifi.text = ""
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func messageBox(_ message: String, withTitle title: String, withAutoDismiss dismiss: Bool){
        let alert: UIAlertController = UIAlertController(title:title, message: message, preferredStyle:  UIAlertController.Style.alert)
        if dismiss == true {
            present(alert, animated: true, completion: {
                DispatchQueue.main.asyncAfter(deadline: .now() + 1.5, execute: {
                    alert.dismiss(animated: false, completion: nil)
                })
            })
         }
        else {
            let defaultAction: UIAlertAction = UIAlertAction(title: "OK", style: UIAlertAction.Style.default, handler:{
                (action: UIAlertAction!) -> Void in
            })
            alert.addAction(defaultAction)
            present(alert, animated: true, completion: nil)
        }
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        
        editText_Address_Wifi.resignFirstResponder()
        return true
    }

    @IBAction func onClick_segmentPortType(_ sender: Any) {
        if segmentPortType.selectedSegmentIndex == 0 {
            editText_Address_Wifi.text = ""
        } else if segmentPortType.selectedSegmentIndex == 2 {
            editText_Address_Wifi.text = ""
        } else {
            editText_Address_Wifi.text = ""
        }
    }
    
    @IBAction func onClick_button_Search(_ sender: Any) {

        //iSearchMode = 0 :SearchCitizenPrinter
        //iSearchMode = 1 :SearchLabelPrinter
        let iSearchMode: Int32 = 0
        var result: Int32 = 0
        var addressArray: Any = []        // Swift 3.0
        var address_Array: [String] = []

        if iSearchMode == 0 {
            
            if segmentPortType.selectedSegmentIndex == 0 {
                addressArray = LabelPrinter.searchCitizenPrinter(CLS_PORT_Bluetooth, withSearchTime: Int32(10), result: &result) as Any
            } else if segmentPortType.selectedSegmentIndex == 2 {
                addressArray = LabelPrinter.searchCitizenPrinter(CLS_PORT_USB, withSearchTime: Int32(10), result: &result) as Any
            } else {
                addressArray = LabelPrinter.searchCitizenPrinter(CLS_PORT_WiFi, withSearchTime: Int32(3), result: &result) as Any
            }

            if (addressArray as AnyObject).count > 0 {
                for i in 0 ..< (addressArray as AnyObject).count {
                    let obj = (addressArray as! NSArray)[i]
                    if let string = obj as? CitizenPrinterInfo {
                        if segmentPortType.selectedSegmentIndex == 0 {
                            address_Array.append(string.bdAddress)
                        } else if segmentPortType.selectedSegmentIndex == 2 {
                            address_Array.append(string.usbSerialNo)
                        }else{
                            address_Array.append(string.ipAddress)
                        }
                    }
                }
            }else{
                messageBox("Search Error : \(result)", withTitle: "0 Search Results", withAutoDismiss: true)
            }
        }else{
            
            if segmentPortType.selectedSegmentIndex == 0 {
                addressArray = LabelPrinter.searchLabelPrinter(CLS_PORT_Bluetooth, withSearchTime: Int32(10), result: &result) as Any
            } else if segmentPortType.selectedSegmentIndex == 2 {
                addressArray = LabelPrinter.searchLabelPrinter(CLS_PORT_USB, withSearchTime: Int32(3), result: &result) as Any
            } else {
                addressArray = LabelPrinter.searchLabelPrinter(CLS_PORT_WiFi, withSearchTime: Int32(3), result: &result) as Any
            }
            
            if (addressArray as AnyObject).count > 0 {
                for i in 0 ..< (addressArray as AnyObject).count {
                    let obj = (addressArray as! NSArray)[i]
                    address_Array.append(obj as! String)
                }
            }else{
                messageBox("Search Error : \(result)", withTitle: "0 Search Results", withAutoDismiss: true)
            }
        }

        if address_Array.count > 0 {
            let alert: UIAlertController = UIAlertController(title:"Select the address", message: nil, preferredStyle:  UIAlertController.Style.alert)
            for i in 0 ..< address_Array.count {
                let defaultAction: UIAlertAction = UIAlertAction(title: address_Array[i], style: UIAlertAction.Style.default, handler:{
                    (action: UIAlertAction!) -> Void in
                    self.editText_Address_Wifi.text = address_Array[i]

                })
                alert.addAction(defaultAction)
            }
            present(alert, animated: true, completion: nil)
        }
    }
    
    @IBAction func onClick_button_LabelPrintSingle(_ sender: Any) {
        
        var ptr: CSJLabelLibSwift.LabelPrinter? = CSJLabelLibSwift.LabelPrinter()
        var result: Int32 = 0

        
        // get address
        let addr = editText_Address_Wifi.text
        
        // connect
        if segmentPortType.selectedSegmentIndex == 0 {
            result = ptr!.connect(CLS_PORT_Bluetooth, withAddrress: addr)
        } else if segmentPortType.selectedSegmentIndex == 2 {
            result = ptr!.connect(CLS_PORT_USB, withAddrress: addr)
        } else {
            result = ptr!.connect(CLS_PORT_WiFi, withAddrress: addr)
        }
        
        if CLS_SUCCESS == result {
            
            result = ptr!.setMeasurementUnit(CLS_UNIT_INCH)
            
            // Create an instance (LabelDesign Class)
            let design = CSJLabelLibSwift.LabelDesign()
            
            // Design label
            DesignLabel_Pattern1(design: design)

            //Print label
            result = ptr!.print(design, quantity: 1)
            
            if result != CLS_SUCCESS {
                messageBox("Print Error : \(result)", withTitle: String().appendingFormat("%d", result), withAutoDismiss: true)
            }

            // discoonect
            _ = ptr!.disconnect()
            
            if result != CLS_SUCCESS {
                messageBox("Transaction Error : \(result)", withTitle: "Error", withAutoDismiss: true)
            }
        } else {
            messageBox("Connect or Printer Error : \(result)", withTitle: "Error", withAutoDismiss: true)
        }
        
        ptr = nil
    }
    
    @IBAction func onClick_button_LabelPrintMulti(_ sender: Any) {
        
        var result: Int32 = 0
        var ptr: CSJLabelLibSwift.LabelPrinter? = CSJLabelLibSwift.LabelPrinter()
        let quantity:Int = 3
        var serialNo:Int = 0

        
        // get address
        let addr = editText_Address_Wifi.text

        // connect
        if segmentPortType.selectedSegmentIndex == 0 {
            result = ptr!.connect(CLS_PORT_Bluetooth, withAddrress: addr)
        } else if segmentPortType.selectedSegmentIndex == 2 {
            result = ptr!.connect(CLS_PORT_USB, withAddrress: addr)
        } else {
            result = ptr!.connect(CLS_PORT_WiFi, withAddrress: addr)
        }
        
        if CLS_SUCCESS == result {
            
            // Set Property (Measurement unit)
            _ = ptr!.setMeasurementUnit(CLS_UNIT_INCH)
            
            //---- Print Design2 ----//
            for i in 1 ..< quantity + 1
            {
                // Create an instance (LabelDesign Class)
                let design = CSJLabelLibSwift.LabelDesign()

                // Design label
                DesignLabel_Pattern2(design: design, serialNo: serialNo)
                
                if (i < quantity)
                {
                    // Set Property (Tear Off)
                    _ = ptr!.setMediaHandling(CLS_MEDIAHANDLING_TEAROFF)
                    
                    // Print design
                    _ = ptr!.print(design, quantity: 1)
                }
                else
                {
                    // Set Property (Cut on last page)
                    _ = ptr!.setMediaHandling(CLS_MEDIAHANDLING_CUT)
                    
                    // Print label
                    _ = ptr!.print(design, quantity: 1)
                }
                
                // Count up serial number
                serialNo += 1
            }
            // discoonect
            _ = ptr!.disconnect()
            
        } else {
            messageBox("Connect or Printer Error : \(result)", withTitle: "Error", withAutoDismiss: true)
        }
        
        ptr = nil
    }
    
    @IBAction func onClick_button_LabelPrint_StatusCheck(_ sender: Any) {
        
        var result: Int32 = 0
        var ptr: CSJLabelLibSwift.LabelPrinter? = CSJLabelLibSwift.LabelPrinter()
        //var iNum:Int32 = 0
        var tResult:(sMessage:String,iStatus:Int32)
        // get address
        let addr = editText_Address_Wifi.text

        // connect
        if segmentPortType.selectedSegmentIndex == 0 {
            result = ptr!.connect(CLS_PORT_Bluetooth, withAddrress: addr)
        } else if segmentPortType.selectedSegmentIndex == 2 {
            result = ptr!.connect(CLS_PORT_USB, withAddrress: addr)
        } else {
            result = ptr!.connect(CLS_PORT_WiFi, withAddrress: addr)
        }
        
        if CLS_SUCCESS == result {
            RunLoop.current.run(until: Date.init(timeIntervalSinceNow: 0.1))

            result = ptr!.printerCheck()
            if (result != CLS_SUCCESS)
            {
                // Error display and disconnect
                example3_printerErrorProc(printer: ptr!, err: ERRORKIND.ERR_PRINTERCHECK, result: Int(result), message: ""
                )
                return
            }
            
            tResult = example3_checkPrinterStatus(printer: ptr!)
            if (tResult.iStatus != 0)
            {
                // Error display and disconnect
                example3_printerErrorProc(printer: ptr!, err: ERRORKIND.ERR_PRINTERSTATUS, result: 0, message: tResult.sMessage)
                return
            }

            _ = example3_print(printer: ptr!)
            if (result != CLS_SUCCESS)
            {
                // Error display and disconnect
                example3_printerErrorProc(printer: ptr!, err: ERRORKIND.ERR_PRINT, result: Int(result), message: "")
                return
            }

            tResult = example3_checkPrinterStatus(printer: ptr!)
            if (tResult.iStatus != 0)
            {
                // Error display and disconnect
                example3_printerErrorProc(printer: ptr!, err: ERRORKIND.ERR_PRINTERSTATUS,result: 0,message: tResult.sMessage)
                return
            }
            else
            {
                messageBox("Print Result : Success", withTitle: "Success", withAutoDismiss: true)

            }

            // discoonect
            _ = ptr!.disconnect()
            
        } else {
            messageBox("Connect or Printer Error : \(result)", withTitle: "Error", withAutoDismiss: true)
        }
        
        ptr = nil

    }
    
    func DesignLabel_Pattern1(design: LabelDesign)
    {
        // Text
        mediumFont(design: design, data: "12 LBS     1 OF 2", x: 230, y: 560)
        
        smallFont(design: design, data: "CITIZEN SYSTEMS AMERICA", x: 10, y: 565)
        smallFont(design: design, data: "1234567890", x: 10, y: 555)
        smallFont(design: design, data: "STE 404", x: 10, y: 545)
        smallFont(design: design, data: "363 VAN NESS WAY", x: 10, y: 535)
        smallFont(design: design, data: "TORRANCE  CA 90501", x: 10, y: 525)
        
        mediumBoldFont(design: design, data: "SHIP", x: 15, y: 500)
        mediumBoldFont(design: design, data: "TO:", x: 19, y: 472)
        
        mediumFont(design: design, data: "LYMAN C SMITH", x: 60, y: 500)
        mediumFont(design: design, data: "9876543210", x: 60, y: 482)
        mediumFont(design: design, data: "SMITH TOWER", x: 60, y: 464)
        mediumFont(design: design, data: "STE 220", x: 60, y: 446)
        mediumFont(design: design, data: "506 2ND AVE", x: 60, y: 428)
        
        mediumBoldFont(design: design, data: "SEATTLE WA 98104-2307", x: 60, y: 407)

        // Line
        _ = design.drawLine(0, y1: 405, x2: 450, y2: 405, thickness: 1)
        _ = design.drawLine(122, y1: 404, x2: 122, y2: 290, thickness: 1)
        _ = design.drawLine(0, y1: 284, x2: 450, y2: 284, thickness: 10)

        // MaxiCode
        let maxicode:[String] = [ "98104", "2307", "840", "001", "1Z123X560120754868" ]
        _ = design.drawMaxiCode(maxicode, rotation: CLS_RT_NORMAL, x: 10, y: 298)
        
        // BarCode
        _ = design.drawBarCode("&D420981042307", symbology: CLS_BCS_CODE128, rotation: CLS_RT_NORMAL, thick: 6, narrow: 3, height: 50, x: 180, y: 320, showText: CLS_BCS_TEXT_HIDE)
        mediumFont(design: design, data: "(420) 98104-2307", x: 200, y: 295)
        
        smallFont(design: design, data: "(420) SHIP TO POSTAL CODE", x: 140, y: 385)

        _ = design.drawTextLocalFont("UPS NEXT DAY AIR", fontName: "ArialMT", rotation: CLS_RT_NORMAL, hRatio: 100, vRatio: 100, point: 20, style: CLS_FNT_BOLD, x: 10, y: 245)
        mediumSmallFont(design: design, data: "TRACKING #: 1Z 123 X56 01 2075 4868",  x: 10, y: 230)
        _ = design.drawTextLocalFont("1", fontName: "ArialMT", rotation: CLS_RT_NORMAL, hRatio: 100, vRatio: 100, point: 42, style: CLS_FNT_BOLD, x: 310, y: 215)

        _ = design.drawLine(0, y1: 225, x2: 450, y2: 225, thickness: 1)
        _ = design.drawBarCode("1Z123X560120754868", symbology: CLS_BCS_CODE128, rotation: CLS_RT_NORMAL, thick: 6, narrow: 3, height: 100, x: 30, y: 105, showText: CLS_BCS_TEXT_HIDE)
        _ = design.drawLine(0, y1: 84, x2: 450, y2: 84, thickness: 10)
 
    }
    
    func DesignLabel_Pattern2(design: LabelDesign, serialNo: Int)
    {
        _ = design.drawTextPtrFont("Product Name: A1B23CD456", locale: CLS_LOCALE_JP, font: CLS_PRT_FNT_TRIUMVIRATE, rotation: CLS_RT_NORMAL, hexp: 1, vexp: 1, size: CLS_PRT_FNT_SIZE_14, x: 10, y: 215)
        
        _ = design.drawBarCode("A1B23CD456", symbology: CLS_BCS_CODE128, rotation: CLS_RT_NORMAL, thick: 6, narrow: 3, height: 60, x: 10, y: 10, showText: CLS_BCS_TEXT_HIDE)
        
        var tmpMsg:String  = "Serial No: "
        tmpMsg += serialNo.description
        
        _ = design.drawTextPtrFont(tmpMsg, locale: CLS_LOCALE_JP, font: CLS_PRT_FNT_TRIUMVIRATE, rotation: CLS_RT_NORMAL, hexp: 1, vexp: 1, size: CLS_PRT_FNT_SIZE_14, x: 10, y: 75)
        
        _ = design.drawBarCode(tmpMsg, symbology: CLS_BCS_CODE128, rotation: CLS_RT_NORMAL, thick: 6, narrow: 3, height: 60, x: 10, y: 150, showText: CLS_BCS_TEXT_HIDE)
    }
    
    func DesignLabel_Pattern3(design: LabelDesign)
    {
        // Order No
        _ = design.drawTextPtrFont("Order No:", locale: CLS_LOCALE_JP, font: CLS_PRT_FNT_TRIUMVIRATE, rotation: CLS_RT_RIGHT90, hexp: 1, vexp: 1, size: CLS_PRT_FNT_SIZE_14, x: 360, y: 280);
        
        // Date
        _ = design.drawTextPtrFont("Date:", locale: CLS_LOCALE_JP, font: CLS_PRT_FNT_TRIUMVIRATE, rotation: CLS_RT_RIGHT90, hexp: 1, vexp: 1, size: CLS_PRT_FNT_SIZE_14, x: 320, y: 280);
        
        // To:
        _ = design.drawTextPtrFont("To:", locale: CLS_LOCALE_JP, font: CLS_PRT_FNT_TRIUMVIRATE, rotation: CLS_RT_RIGHT90, hexp: 1, vexp: 1, size: CLS_PRT_FNT_SIZE_14, x: 280, y: 280);
        
        // From:
        _ = design.drawTextPtrFont("From:", locale: CLS_LOCALE_JP, font: CLS_PRT_FNT_TRIUMVIRATE, rotation: CLS_RT_RIGHT90, hexp: 1, vexp: 1, size: CLS_PRT_FNT_SIZE_14, x: 240, y: 280);
        
        // Rectangle
        _ = design.drawRect(230, y: 5, width: 155, height: 280, thickness: 1);
        
        // Order No
        _ = design.drawTextPtrFont("Order No:", locale: CLS_LOCALE_JP, font: CLS_PRT_FNT_TRIUMVIRATE, rotation: CLS_RT_RIGHT90, hexp: 1, vexp: 1, size: CLS_PRT_FNT_SIZE_14, x: 140, y: 280);
        
        // Date
        _ = design.drawTextPtrFont("Date:", locale: CLS_LOCALE_JP, font: CLS_PRT_FNT_TRIUMVIRATE, rotation: CLS_RT_RIGHT90, hexp: 1, vexp: 1, size: CLS_PRT_FNT_SIZE_14, x: 100, y: 280);
        
        // To:
        _ = design.drawTextPtrFont("To:", locale: CLS_LOCALE_JP, font: CLS_PRT_FNT_TRIUMVIRATE, rotation: CLS_RT_RIGHT90, hexp: 1, vexp: 1, size: CLS_PRT_FNT_SIZE_14, x: 60, y: 280);
        
        // From:
        _ = design.drawTextPtrFont("From:", locale: CLS_LOCALE_JP, font: CLS_PRT_FNT_TRIUMVIRATE, rotation: CLS_RT_RIGHT90, hexp: 1, vexp: 1, size: CLS_PRT_FNT_SIZE_14, x: 20, y: 280);
        
        // Rectangle
        _ = design.drawRect(10, y: 5, width: 155, height: 280, thickness: 1);
    }
    
    func smallFont(design: LabelDesign, data: String , x: Int32, y: Int32)
    {
        _ = design.drawTextPtrFont(data,  //string data
            locale: CLS_LOCALE_OTHER,      //int locale
            font: CLS_PRT_FNT_TRIUMVIRATE, //int font
            rotation: CLS_RT_NORMAL,       //int rotation
            hexp: 1,                       //int hexp
            vexp: 1,                       //int vexp
            size: CLS_PRT_FNT_SIZE_8,      //int size
            x: x,                          //int x
            y: y)                          //int y
    }
    
    func mediumSmallFont(design: LabelDesign, data: String , x: Int32, y: Int32)
    {
        _ = design.drawTextPtrFont(data,  //string data
            locale: CLS_LOCALE_OTHER,      //int locale
            font: CLS_PRT_FNT_TRIUMVIRATE, //int font
            rotation: CLS_RT_NORMAL,       //int rotation
            hexp: 1,                       //int hexp
            vexp: 1,                       //int vexp
            size: CLS_PRT_FNT_SIZE_10,     //int size
            x: x,                          //int x
            y: y)                          //int y
    }
    
    func mediumFont(design: LabelDesign, data: String , x: Int32, y: Int32)
    {
        _ = design.drawTextPtrFont( data, //string data
            locale: CLS_LOCALE_OTHER,      //int locale
            font: CLS_PRT_FNT_TRIUMVIRATE, //int font
            rotation: CLS_RT_NORMAL,       //int rotation
            hexp: 1,                       //int hexp
            vexp: 1,                       //int vexp
            size: CLS_PRT_FNT_SIZE_12,     //int size
            x: x,                          //int x
            y: y)                          //int y
    }
    
    func mediumBoldFont(design: LabelDesign, data: String , x: Int32, y: Int32)
    {
        _ = design.drawTextPtrFont( data,      //string data
            locale: CLS_LOCALE_OTHER,           //int locale
            font: CLS_PRT_FNT_TRIUMVIRATE_B,    //int font
            rotation: CLS_RT_NORMAL,            //int rotation
            hexp: 1,                            //int hexp
            vexp: 1,                            //int vexp
            size: CLS_PRT_FNT_SIZE_14,          //int size
            x: x,                               //int x
            y: y)                               //int y
    }
    
    func example3_checkPrinterStatus(printer: LabelPrinter) -> (sMessage:String, iStatus: Int32)
    {
        var retMsg:String = ""
        var iReturnValue:Int32
        var iStatus:Int32 = 0

        // Check printer status
        retMsg.append("Printer Status")
        retMsg = retMsg + "\n"
        iReturnValue = printer.getCommandInterpreterInAction()
        retMsg.append("\(iReturnValue): Command interpreter in action")
        retMsg = retMsg + "\n"
        iStatus += iReturnValue

        iReturnValue = printer.getPaperError();
        retMsg.append("\(iReturnValue): Paper error")
        retMsg = retMsg + "\n"
        iStatus += iReturnValue

        iReturnValue = printer.getRibbonEnd();
        retMsg.append("\(iReturnValue): Ribbon end")
        retMsg = retMsg + "\n"
        iStatus += iReturnValue

        iReturnValue = printer.getBatchProcessing();
        retMsg.append("\(iReturnValue): Batch processing")
        retMsg = retMsg + "\n"
        iStatus += iReturnValue

        iReturnValue = printer.getPrinting();
        retMsg.append("\(iReturnValue): Printing")
        retMsg = retMsg + "\n"
        iStatus += iReturnValue

        iReturnValue = printer.getPause();
        retMsg.append("\(iReturnValue): Pause")
        retMsg = retMsg + "\n"
        iStatus += iReturnValue

        iReturnValue = printer.getWaitingForPeeling();
        retMsg.append("\(iReturnValue): Waiting for peeling")
        retMsg = retMsg + "\n"
        iStatus += iReturnValue

        return (retMsg, iStatus)
    }
    
    func example3_print(printer: LabelPrinter) -> Int32
    {
        var ret:Int32 = CLS_SUCCESS
        
        // Set Property (Measurement unit)
        _ = printer.setMeasurementUnit(CLS_UNIT_INCH);
        
        // Create an instance (LabelDesign Class)
        let design = CSJLabelLibSwift.LabelDesign()

        // Design label
        DesignLabel_Pattern3(design: design);
        
        // Print label
        ret = printer.print(design, quantity: 2);
        if (ret != CLS_SUCCESS)
        {
            return ret;
        }
        
        RunLoop.current.run(until: Date.init(timeIntervalSinceNow: 0.5))

        // Check printing
        while (true) {
            // Get the printer status
            ret = printer.printerCheck();
            if (ret != CLS_SUCCESS) {
                return ret;
            }
            
            // Checks the Printing and BatchProcessing populates
            if ((printer.getPrinting() == 0) && (printer.getBatchProcessing() == 0)) {
                break;
            }
        }
        
        return ret;
    }

    func example3_printerErrorProc( printer: LabelPrinter, err errKind: ERRORKIND, result: Int, message msg: String)
    {
        var tmpMsg:String = ""

        if (errKind == ERRORKIND.ERR_CONNECT)
        {
            messageBox("Connect failure : \(result)", withTitle: "Error", withAutoDismiss: true)
        }
        else if (errKind == ERRORKIND.ERR_PRINTERCHECK)
        {
            messageBox("PrinterCheck failure : \(result)", withTitle: "Error", withAutoDismiss: true)
            _ = printer.disconnect();
        }
        else if (errKind == ERRORKIND.ERR_PRINT)
        {
            messageBox("Print failure : \(result)", withTitle: "Error", withAutoDismiss: true)
            _ = printer.disconnect();
        }
        else if (errKind == ERRORKIND.ERR_PRINTERSTATUS)
        {
            tmpMsg.append("The printer can not print.")
            tmpMsg = tmpMsg + "\n"
            tmpMsg.append("Select \"Yes\" to reset the printer, or \"No\" if you want to restore it manually.")
            tmpMsg = tmpMsg + "\n"
            tmpMsg.append("*Clear print buffer is select \"Yes\".")
            tmpMsg = tmpMsg + "\n\n"
            tmpMsg.append(msg)
            
            // Show error dialog
            let alert: UIAlertController = UIAlertController(title:"Printer Error", message: tmpMsg, preferredStyle:  UIAlertController.Style.alert)
            let defaultAction: UIAlertAction = UIAlertAction(title: "Yes", style: UIAlertAction.Style.default, handler:{
                (action: UIAlertAction!) -> Void in
                _ = printer.clearOutput();
                _ = printer.disconnect();
                
            })
            
            let cancelAction:UIAlertAction = UIAlertAction(title: "No", style: UIAlertAction.Style.cancel, handler:{
                (action:UIAlertAction!) -> Void in
                 _ = printer.disconnect();
            })
            alert.addAction(defaultAction)
            alert.addAction(cancelAction)
            
            present(alert, animated: true, completion: nil)
            
        }
        else
        {
            // No Operation
        }
    }
}

